"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
var inversify_1 = require("inversify");
var path = require("path");
var force_error_impl_1 = require("./force-error-impl");
var blackHoleStream = new (require('black-hole-stream'))();
var fs = require('fs');
var CommandUtilImpl = (function (_super) {
    __extends(CommandUtilImpl, _super);
    function CommandUtilImpl(postal) {
        var _this = _super.call(this) || this;
        _this.postal = postal;
        _this._console = {
            log: _this.stdoutLog.bind(_this),
            info: _this.stdoutLog.bind(_this),
            warn: _this.stdoutLog.bind(_this),
            error: _this.stderrLog.bind(_this)
        };
        _this.registerProcessManagementEvents();
        var cacheStdoutWrite = process.stdout.write;
        var cacheStderrWrite = process.stderr.write;
        _this.postal.subscribe({
            channel: 'CommandUtil',
            topic: 'SuppressConsoleOutput',
            callback: function (data) {
                if (data.suppressConsoleOutput) {
                    process.stdout.write = process.stderr.write = blackHoleStream.write.bind(blackHoleStream);
                }
                else {
                    process.stdout.write = cacheStdoutWrite;
                    process.stderr.write = cacheStderrWrite;
                }
            }
        });
        _this.postal.subscribe({
            channel: 'CommandUtil',
            topic: 'ProgressBarStarted',
            callback: function (data) {
                var dataConsole = data.console;
                _this._console.log = dataConsole.log;
                _this._console.info = dataConsole.info;
                _this._console.warn = dataConsole.warn;
                _this._console.error = dataConsole.error;
            }
        });
        return _this;
    }
    CommandUtilImpl.prototype.exitHandler = function (options, err) {
        if (options.cleanup) {
        }
        if (err) {
            if (err && err.message) {
                this.stderrLog(err.message);
            }
        }
        if (options.exit) {
            process.exit();
        }
    };
    CommandUtilImpl.prototype.registerProcessManagementEvents = function () {
        process.stdin.resume();
        process.on('exit', this.exitHandler.bind(this, { cleanup: true }));
        process.on('SIGINT', this.exitHandler.bind(this, { exit: true }));
        process.on('uncaughtException', this.exitHandler.bind(this, { exit: true }));
    };
    CommandUtilImpl.prototype.stderrWrite = function (msg) {
        process.stderr.write(msg);
    };
    CommandUtilImpl.prototype.stdoutWrite = function (msg) {
        process.stdout.write(msg);
    };
    CommandUtilImpl.prototype.stderrLog = function (msg) {
        this.stderrWrite(msg + "\n");
    };
    CommandUtilImpl.prototype.stdoutLog = function (msg) {
        this.stdoutWrite(msg + "\n");
    };
    CommandUtilImpl.prototype.returnErrorStringOrMessage = function (err, message) {
        var errorMessage = this.logError(err, false);
        return errorMessage.length ? errorMessage : message;
    };
    CommandUtilImpl.prototype.error = function (msg) {
        this._console.error(msg);
    };
    CommandUtilImpl.prototype.log = function (msg) {
        this._console.log(msg);
    };
    CommandUtilImpl.prototype.logErrors = function (errs, writeErrorToConsole) {
        var _this = this;
        if (writeErrorToConsole === void 0) { writeErrorToConsole = true; }
        var retVal = [];
        errs.forEach(function (err) {
            retVal.push(_this.logError(err, writeErrorToConsole));
        });
        return retVal;
    };
    CommandUtilImpl.prototype.logError = function (err, writeErrorToConsole) {
        if (writeErrorToConsole === void 0) { writeErrorToConsole = true; }
        if (err) {
            if (writeErrorToConsole) {
                this._console.error(err.message);
            }
            return err.message;
        }
        return '';
    };
    CommandUtilImpl.prototype.processExitIfError = function (err) {
        if (err) {
            this.processExitWithError(err);
        }
    };
    CommandUtilImpl.prototype.processExitWithError = function (err, nonErrorMessage) {
        if (nonErrorMessage === void 0) { nonErrorMessage = null; }
        this.processExit(!!err ? 1 : 0, !!err ? err.message : !!nonErrorMessage ? nonErrorMessage : '');
    };
    CommandUtilImpl.prototype.processExit = function (exitCode, msg) {
        if (exitCode === void 0) { exitCode = 0; }
        if (msg === void 0) { msg = null; }
        this._console.log(!!msg ? msg : '');
        process.exit(exitCode);
    };
    CommandUtilImpl.prototype.callbackIfError = function (cb, err, result) {
        if (err === void 0) { err = null; }
        if (result === void 0) { result = null; }
        cb = this.checkCallback(cb);
        err && cb(err, result);
        return !!err;
    };
    CommandUtilImpl.prototype.logAndCallback = function (msg, cb, err, result) {
        if (err === void 0) { err = null; }
        if (result === void 0) { result = null; }
        this._console.log(err ? err.message : msg);
        cb = this.checkCallback(cb);
        cb(err, result);
        return !!err;
    };
    CommandUtilImpl.prototype.getConfigFilePath = function (filename, extension) {
        if (extension === void 0) { extension = '.json'; }
        var regex = new RegExp('(.*)\\' + extension + '$', 'i');
        var cwd = process.cwd();
        filename = regex.test(filename)
            ? filename.replace(regex, '$1' + extension)
            : filename + extension;
        return path.resolve(cwd, filename);
    };
    CommandUtilImpl = __decorate([
        inversify_1.injectable(),
        __param(0, inversify_1.inject('IPostal')),
        __metadata("design:paramtypes", [Object])
    ], CommandUtilImpl);
    return CommandUtilImpl;
}(force_error_impl_1.ForceErrorImpl));
exports.CommandUtilImpl = CommandUtilImpl;
//# sourceMappingURL=command-util-impl.js.map