"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
var request = require("request");
var inversify_1 = require("inversify");
var path = require("path");
var fs = require("fs");
var nodeUrl = require("url");
var _ = require("lodash");
var force_error_impl_1 = require("./force-error-impl");
var fileExists = require('file-exists');
var async = require('async');
var RemoteCatalogGetterImpl = (function (_super) {
    __extends(RemoteCatalogGetterImpl, _super);
    function RemoteCatalogGetterImpl(commandUtil, safeJson) {
        var _this = _super.call(this) || this;
        _this.commandUtil = commandUtil;
        _this.safeJson = safeJson;
        return _this;
    }
    RemoteCatalogGetterImpl.prototype.getCatalogFromUrl = function (url, cb) {
        var me = this;
        cb = me.checkCallback(cb);
        if (me.checkForceError('RemoteCatalogGetterImpl.getCatalogFromUrl', cb)) {
            return;
        }
        me.getParsedUrl(url, function (err, parsedUrl) {
            if (me.commandUtil.callbackIfError(cb, err)) {
                return;
            }
            var baseUrl = (parsedUrl.protocol)
                ? parsedUrl.protocol + "//" + parsedUrl.hostname + path.dirname(parsedUrl.path)
                : path.dirname(parsedUrl.path);
            me.getRemoteResource(parsedUrl, 'root', function (err, remoteCatalogResource) {
                if (me.commandUtil.callbackIfError(cb, err)) {
                    return;
                }
                try {
                    var remoteCatalog_1 = remoteCatalogResource.parsedObject;
                    var fnArray_1 = [];
                    async.each(remoteCatalog_1.entries, function (entry, cb) {
                        async.each(entry.urls, function (url, cb) {
                            me.getParsedUrl(url, function (err, parsedUrl) {
                                if (me.commandUtil.callbackIfError(cb, err)) {
                                    return;
                                }
                                if (!parsedUrl.protocol) {
                                    url = path.isAbsolute(parsedUrl.path) ? parsedUrl.path : baseUrl + "/" + parsedUrl.path;
                                }
                                fnArray_1.push(async.apply(me.getRemoteResource.bind(me), url, entry.name));
                            });
                            cb();
                        }, function (err) {
                            me.commandUtil.logError(err);
                            cb();
                        });
                    }, function (err) {
                        if (me.commandUtil.callbackIfError(cb, err)) {
                            return;
                        }
                        async.parallel(fnArray_1, function (err, results) {
                            if (me.commandUtil.callbackIfError(cb, err)) {
                                return;
                            }
                            remoteCatalog_1.entries.forEach(function (entry) {
                                entry.resources =
                                    _.filter(results, { parentCatalogEntryName: entry.name });
                            });
                            cb(null, remoteCatalog_1);
                        });
                    });
                }
                catch (err) {
                    me.commandUtil.callbackIfError(cb, err);
                }
            });
        });
    };
    RemoteCatalogGetterImpl.prototype.getRemoteResource = function (url, parentCatalogEntryName, cb) {
        var me = this;
        cb = me.checkCallback(cb);
        if (me.checkForceError('RemoteCatalogGetterImpl.getRemoteResource', cb)) {
            return;
        }
        me.resolveTextResourceFromUrl(url, function (err, text, absoluteUrl) {
            if (me.commandUtil.callbackIfError(cb, err)) {
                return;
            }
            me.safeJson.safeParse(text, function (err, parsedObject) {
                var name = path.basename(absoluteUrl);
                cb(err, {
                    absoluteUrl: absoluteUrl, name: name, text: text, parsedObject: parsedObject, parentCatalogEntryName: parentCatalogEntryName
                });
            });
        });
    };
    RemoteCatalogGetterImpl.prototype.resolveJsonObjectFromUrl = function (url, cb) {
        var me = this;
        cb = me.checkCallback(cb);
        if (me.checkForceError('RemoteCatalogGetterImpl.resolveJsonObjectFromUrl', cb)) {
            return;
        }
        me.resolveTextResourceFromUrl(url, function (err, text, absoluteUrl) {
            if (me.commandUtil.callbackIfError(cb, err, false)) {
                return;
            }
            me.safeJson.safeParse(text, function (err, jsonObject) {
                cb(err, jsonObject, absoluteUrl);
            });
        });
    };
    RemoteCatalogGetterImpl.prototype.resolveTextResourceFromUrl = function (url, cb) {
        var me = this;
        cb = me.checkCallback(cb);
        if (me.checkForceError('RemoteCatalogGetterImpl.resolveTextResourceFromUrl', cb)) {
            return;
        }
        me.getParsedUrl(url, function (err, parsedUrl) {
            if (err) {
                return cb(err, '', null);
            }
            try {
                if (!parsedUrl.protocol) {
                    var urlString_1 = parsedUrl.path;
                    fileExists(urlString_1, function (err, exists) {
                        if (me.commandUtil.callbackIfError(cb, err, false)) {
                            return;
                        }
                        if (exists) {
                            fs.readFile(urlString_1, 'utf8', function (err, data) {
                                if (me.commandUtil.callbackIfError(cb, err)) {
                                    return;
                                }
                                cb(null, data.toString(), urlString_1);
                            });
                            return;
                        }
                        cb(new Error("Url: '" + url + "' does not exist"), '', urlString_1);
                    });
                    return;
                }
                request(parsedUrl.href, function (err, res, text) {
                    if (err) {
                        return cb(err, '', parsedUrl.href);
                    }
                    if (res.statusCode !== 200) {
                        return cb(new Error("Error retrieving '" + parsedUrl.href + "'"), '', parsedUrl.href);
                    }
                    return cb(null, text.toString(), parsedUrl.href);
                });
            }
            catch (err) {
                return cb(err, null);
            }
        });
    };
    RemoteCatalogGetterImpl.prototype.getParsedUrl = function (url, cb) {
        var me = this;
        cb = me.checkCallback(cb);
        if (me.checkForceError('RemoteCatalogGetterImpl.getParsedUrl', cb)) {
            return;
        }
        try {
            if (url.constructor.name === 'Url') {
                return cb(null, url);
            }
            if (url.constructor.name === 'String') {
                if (!url) {
                    return cb(new Error('Empty string is not a url'));
                }
                var parsedUrl = nodeUrl.parse(url);
                return cb(null, parsedUrl);
            }
        }
        catch (err) {
            cb(err);
        }
    };
    RemoteCatalogGetterImpl = __decorate([
        __param(0, inversify_1.inject('CommandUtil')),
        __param(1, inversify_1.inject('SafeJson')),
        __metadata("design:paramtypes", [Object, Object])
    ], RemoteCatalogGetterImpl);
    return RemoteCatalogGetterImpl;
}(force_error_impl_1.ForceErrorImpl));
exports.RemoteCatalogGetterImpl = RemoteCatalogGetterImpl;
//# sourceMappingURL=remote-catalog-impl.js.map