"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
var inversify_config_1 = require("../inversify.config");
var chai_1 = require("chai");
var sinon = require("sinon");
describe('CommandLine', function () {
    var commandLine;
    var testCommandArray = [];
    var testCommandArrayArray = [];
    before(function () {
        commandLine = inversify_config_1.default.get('CommandLine');
    });
    describe('create using kernel', function () {
        it('should be created by kernel', function (done) {
            chai_1.expect(commandLine).to.not.equal(null);
            done();
        });
    });
    before(function () {
        commandLine = inversify_config_1.default.get('CommandLine');
    });
    describe('printTable', function () {
        var consoleTableStub;
        var testRows = ['row1', 'row2'];
        before(function () {
            consoleTableStub = sinon.stub(console, 'table', function (rows) {
                chai_1.expect(rows).to.equal(testRows);
            });
        });
        it('should call printTable', function (done) {
            commandLine.printTable(testRows);
            chai_1.expect(consoleTableStub.called).to.equal(true);
            done();
        });
        after(function () {
            console.table.restore();
        });
    });
    before(function () {
        commandLine = inversify_config_1.default.get('CommandLine');
    });
    describe('CommandLine.addCommand', function () {
        before(function () {
            commandLine.init({
                version: function () {
                    return 'version_number';
                }
            });
            commandLine.addCommand(testCommand(testCommandArrayArray, function (argv) {
                for (var i = 0; i < argv._.length; ++i) {
                    chai_1.expect(argv._[i]).to.equal(testCommandArray[i]);
                }
            }));
        });
        describe('commandLine.exec()', function () {
            it('commands invoked correct functions', function (done) {
                testCommandArrayArray.forEach(function (command) {
                    commandLine.exec(testCommandArray = command);
                });
                done();
            });
        });
    });
});
function testCommand(commands, cb) {
    var newCommand = inversify_config_1.default.get('CommandImpl');
    newCommand.aliases = ['alias-top-1', 'alias-top-2'];
    newCommand.commandDesc = 'topCommandDesc';
    for (var i = 0; i < 3; ++i) {
        var newSubCommand = inversify_config_1.default.get('CommandImpl');
        newSubCommand.aliases = ['alias-sub-1-1-' + i, 'alias-sub-1-2-' + i];
        newSubCommand.commandDesc = 'subCommandDesc-' + i;
        if (i & 1) {
            commands.push([newCommand.aliases[0], newSubCommand.aliases[0]]);
            newSubCommand.handler = cb;
        }
        else {
            for (var j = 0; j < 3; ++j) {
                var newSubSubCommand = inversify_config_1.default.get('CommandImpl');
                newSubSubCommand.aliases = ['alias-sub-1-1-' + i + '-' + j, 'alias-sub-1-2-' + i + '-' + j];
                newSubSubCommand.commandDesc = 'subCommandDesc-' + i + '-' + j;
                commands.push([newCommand.aliases[0], newSubCommand.aliases[0], newSubSubCommand.aliases[0]]);
                newSubSubCommand.handler = cb;
                newSubCommand.subCommands.push(newSubSubCommand);
            }
        }
        newCommand.subCommands.push(newSubCommand);
    }
    return newCommand;
}
//# sourceMappingURL=command-line.test.js.map