"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
var inversify_config_1 = require("../inversify.config");
var chai_1 = require("chai");
var path = require("path");
var Url = require('url');
describe('Testing RemoteCatalogGetter Creation/Force Error', function () {
    var remoteCatalogGetter;
    beforeEach(function () {
        remoteCatalogGetter = inversify_config_1.default.get('RemoteCatalogGetter');
    });
    afterEach(function () {
        remoteCatalogGetter.forceError = false;
    });
    it('should be created by kernel', function (done) {
        chai_1.expect(remoteCatalogGetter).to.exist;
        done();
    });
});
describe('Testing RemoteCatalogGetter.parsedUrl', function () {
    var remoteCatalogGetter;
    beforeEach(function () {
        remoteCatalogGetter = inversify_config_1.default.get('RemoteCatalogGetter');
    });
    afterEach(function () {
        remoteCatalogGetter.forceError = false;
    });
    it('should have callback with error', function (done) {
        remoteCatalogGetter.forceError = true;
        remoteCatalogGetter.getParsedUrl(null, function (err, parsedUrl) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal('force error: RemoteCatalogGetterImpl.getParsedUrl');
            chai_1.expect(parsedUrl).to.not.exist;
            done();
        });
    });
    it('should have callback with error when url is undefined', function (done) {
        var url;
        remoteCatalogGetter.getParsedUrl(url, function (err, parsedUrl) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal("Cannot read property 'constructor' of undefined");
            chai_1.expect(parsedUrl).to.not.exist;
            done();
        });
    });
    it('should have callback with error when url is null', function (done) {
        var url = null;
        remoteCatalogGetter.getParsedUrl(url, function (err, parsedUrl) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal("Cannot read property 'constructor' of null");
            chai_1.expect(parsedUrl).to.not.exist;
            done();
        });
    });
    it('should have callback with error when url is and empty string', function (done) {
        var url = '';
        remoteCatalogGetter.getParsedUrl(url, function (err, parsedUrl) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal('Empty string is not a url');
            chai_1.expect(parsedUrl).to.not.exist;
            done();
        });
    });
    it('should return parsed web url', function (done) {
        var url = 'http://www.yahoo.com/grindel.jpg';
        remoteCatalogGetter.getParsedUrl(url, function (err, parsedUrl) {
            chai_1.expect(err).to.not.exist;
            chai_1.expect(parsedUrl).to.be.an.instanceof(Url.constructor);
            chai_1.expect(parsedUrl.protocol).to.equal('http:');
            done();
        });
    });
    it('should return parsed file url', function (done) {
        var url = '/tmp/grindel.jpg';
        remoteCatalogGetter.getParsedUrl(url, function (err, parsedUrl) {
            chai_1.expect(err).to.not.exist;
            chai_1.expect(parsedUrl).to.be.an.instanceof(Url.constructor);
            chai_1.expect(parsedUrl.protocol).to.not.exist;
            done();
        });
    });
});
describe('Testing RemoteCatalogGetter.resolveTextResourceFromUrl', function () {
    var remoteCatalogGetter;
    beforeEach(function () {
        remoteCatalogGetter = inversify_config_1.default.get('RemoteCatalogGetter');
    });
    afterEach(function () {
        remoteCatalogGetter.forceError = false;
    });
    it('should have callback with error', function (done) {
        remoteCatalogGetter.forceError = true;
        remoteCatalogGetter.resolveTextResourceFromUrl(null, function (err, text, absoluteUrl) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal('force error: RemoteCatalogGetterImpl.resolveTextResourceFromUrl');
            chai_1.expect(absoluteUrl).to.not.exist;
            done();
        });
    });
    it('should have callback with error when url is undefined', function (done) {
        var url;
        remoteCatalogGetter.resolveTextResourceFromUrl(url, function (err, text, absoluteUrl) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal("Cannot read property 'constructor' of undefined");
            chai_1.expect(absoluteUrl).to.not.exist;
            done();
        });
    });
    it('should have callback with error when url is null', function (done) {
        var url = null;
        remoteCatalogGetter.resolveTextResourceFromUrl(url, function (err, text, absoluteUrl) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal("Cannot read property 'constructor' of null");
            chai_1.expect(absoluteUrl).to.not.exist;
            done();
        });
    });
    it('should have callback with error when url is and empty string', function (done) {
        var url = '';
        remoteCatalogGetter.resolveTextResourceFromUrl(url, function (err, text, absoluteUrl) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal('Empty string is not a url');
            chai_1.expect(absoluteUrl).to.not.exist;
            done();
        });
    });
    it("should return non-null error if file doesn't exist", function (done) {
        var cwd = process.cwd();
        var url = path.resolve(cwd, 'ts/test/json/valid__.json');
        remoteCatalogGetter.resolveTextResourceFromUrl(url, function (err, text, absoluteUrl) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal("Url: '/home/jreeme/src/firmament-yargs/ts/test/json/valid__.json' does not exist");
            chai_1.expect(absoluteUrl).to.equal(url);
            chai_1.expect(text).to.be.empty;
            done();
        });
    });
    it("should return null error if file exists and text should be '12345'", function (done) {
        var cwd = process.cwd();
        var url = path.resolve(cwd, 'ts/test/json/exists.json');
        remoteCatalogGetter.resolveTextResourceFromUrl(url, function (err, text, absoluteUrl) {
            chai_1.expect(err).to.not.exist;
            chai_1.expect(absoluteUrl).to.equal(url);
            chai_1.expect(text).to.equal('12345\n');
            done();
        });
    });
    it("should return non-null error if web resource doesn't exist", function (done) {
        var url = 'http://www.nositeatall.com/gumdrop.jpg';
        remoteCatalogGetter.resolveTextResourceFromUrl(url, function (err, text, absoluteUrl) {
            chai_1.expect(err).to.exist;
            chai_1.expect(absoluteUrl).to.equal(url);
            chai_1.expect(text).to.be.empty;
            done();
        });
    });
    it("should return null error if web resource exists", function (done) {
        var url = 'http://www.yahoo.com/';
        remoteCatalogGetter.resolveTextResourceFromUrl(url, function (err, text, absoluteUrl) {
            chai_1.expect(err).to.not.exist;
            chai_1.expect(absoluteUrl).to.equal(url);
            chai_1.expect(text).to.be.not.empty;
            done();
        });
    });
});
describe('Testing RemoteCatalogGetter.resolveJsonObjectFromUrl', function () {
    var remoteCatalogGetter;
    beforeEach(function () {
        remoteCatalogGetter = inversify_config_1.default.get('RemoteCatalogGetter');
    });
    afterEach(function () {
        remoteCatalogGetter.forceError = false;
    });
    it('should have callback with error', function (done) {
        remoteCatalogGetter.forceError = true;
        remoteCatalogGetter.resolveJsonObjectFromUrl(null, function (err, jsonObject, absoluteUrl) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal('force error: RemoteCatalogGetterImpl.resolveJsonObjectFromUrl');
            chai_1.expect(absoluteUrl).to.not.exist;
            done();
        });
    });
    it('should have callback with error when url is undefined', function (done) {
        var url;
        remoteCatalogGetter.resolveJsonObjectFromUrl(url, function (err, jsonObject, absoluteUrl) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal("Cannot read property 'constructor' of undefined");
            chai_1.expect(absoluteUrl).to.not.exist;
            done();
        });
    });
    it('should have callback with error when url is null', function (done) {
        var url = null;
        remoteCatalogGetter.resolveJsonObjectFromUrl(url, function (err, jsonObject, absoluteUrl) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal("Cannot read property 'constructor' of null");
            chai_1.expect(absoluteUrl).to.not.exist;
            done();
        });
    });
    it('should return error on bad JSON', function (done) {
        var cwd = process.cwd();
        var url = path.resolve(cwd, 'ts/test/json/bad-json.json');
        remoteCatalogGetter.resolveJsonObjectFromUrl(url, function (err, jsonObject, absoluteUrl) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal('Unexpected token : in JSON at position 1');
            chai_1.expect(absoluteUrl).to.equal(url);
            done();
        });
    });
    it('should return ', function (done) {
        var cwd = process.cwd();
        var url = path.resolve(cwd, 'ts/test/json/valid.json');
        remoteCatalogGetter.resolveJsonObjectFromUrl(url, function (err, jsonObject, absoluteUrl) {
            chai_1.expect(err).to.not.exist;
            chai_1.expect(jsonObject).to.include({ description: 'Command Group 0' });
            chai_1.expect(absoluteUrl).to.equal(url);
            done();
        });
    });
});
describe('Testing RemoteCatalogGetter.getRemoteResource', function () {
    var remoteCatalogGetter;
    beforeEach(function () {
        remoteCatalogGetter = inversify_config_1.default.get('RemoteCatalogGetter');
    });
    afterEach(function () {
        remoteCatalogGetter.forceError = false;
    });
    it('should have callback with error', function (done) {
        remoteCatalogGetter.forceError = true;
        remoteCatalogGetter.getRemoteResource(null, null, function (err, remoteCatalogResource) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal('force error: RemoteCatalogGetterImpl.getRemoteResource');
            chai_1.expect(remoteCatalogResource).to.not.exist;
            done();
        });
    });
    it('should have callback with error when url is undefined', function (done) {
        var url;
        remoteCatalogGetter.getRemoteResource(url, null, function (err, remoteCatalogResource) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal("Cannot read property 'constructor' of undefined");
            chai_1.expect(remoteCatalogResource).to.not.exist;
            done();
        });
    });
    it('should have callback with error when url is null', function (done) {
        var url = null;
        remoteCatalogGetter.getRemoteResource(url, null, function (err, remoteCatalogResource) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal("Cannot read property 'constructor' of null");
            chai_1.expect(remoteCatalogResource).to.not.exist;
            done();
        });
    });
    it('should have callback with error when json resource does not exist', function (done) {
        var cwd = process.cwd();
        var nonExistentJsonFile = 'valid__.json';
        var url = path.resolve(cwd, 'ts/test/json', nonExistentJsonFile);
        var parentCatalogEntryName = 'The parent catalog entry name!';
        remoteCatalogGetter.getRemoteResource(url, parentCatalogEntryName, function (err, remoteCatalogResource) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal("Url: '/home/jreeme/src/firmament-yargs/ts/test/json/valid__.json' does not exist");
            chai_1.expect(remoteCatalogResource).to.not.exist;
            done();
        });
    });
    it('should have callback with error when resource does not contain valid json', function (done) {
        var cwd = process.cwd();
        var nonExistentJsonFile = 'bad-json.json';
        var url = path.resolve(cwd, 'ts/test/json', nonExistentJsonFile);
        var parentCatalogEntryName = 'The parent catalog entry name!';
        remoteCatalogGetter.getRemoteResource(url, parentCatalogEntryName, function (err, remoteCatalogResource) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal('Unexpected token : in JSON at position 1');
            chai_1.expect(remoteCatalogResource).to.exist;
            done();
        });
    });
    it('should provide reasonable remoteCatalogResource when given link to valid.json', function (done) {
        var cwd = process.cwd();
        var validJson = 'valid.json';
        var url = path.resolve(cwd, 'ts/test/json', validJson);
        var parentCatalogEntryName = 'The parent catalog entry name!';
        remoteCatalogGetter.getRemoteResource(url, parentCatalogEntryName, function (err, remoteCatalogResource) {
            chai_1.expect(err).to.not.exist;
            chai_1.expect(remoteCatalogResource).to.exist;
            chai_1.expect(remoteCatalogResource).to.include({
                name: validJson
            });
            chai_1.expect(remoteCatalogResource.parentCatalogEntryName).to.equal(parentCatalogEntryName);
            done();
        });
    });
});
describe('Testing RemoteCatalogGetter.getCatalogFromUrl', function () {
    var remoteCatalogGetter;
    beforeEach(function () {
        remoteCatalogGetter = inversify_config_1.default.get('RemoteCatalogGetter');
    });
    afterEach(function () {
        remoteCatalogGetter.forceError = false;
    });
    it('should have callback with error', function (done) {
        remoteCatalogGetter.forceError = true;
        remoteCatalogGetter.getCatalogFromUrl(null, function (err, remoteCatalog) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal('force error: RemoteCatalogGetterImpl.getCatalogFromUrl');
            chai_1.expect(remoteCatalog).to.not.exist;
            done();
        });
    });
    it('should have callback with error when url is undefined', function (done) {
        var url;
        remoteCatalogGetter.getCatalogFromUrl(url, function (err, remoteCatalog) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal("Cannot read property 'constructor' of undefined");
            chai_1.expect(remoteCatalog).to.not.exist;
            done();
        });
    });
    it('should have callback with error when url is null', function (done) {
        var url = null;
        remoteCatalogGetter.getCatalogFromUrl(url, function (err, remoteCatalog) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal("Cannot read property 'constructor' of null");
            chai_1.expect(remoteCatalog).to.not.exist;
            done();
        });
    });
    it('should have callback with error when json resource does not exist', function (done) {
        var cwd = process.cwd();
        var nonExistentJsonFile = 'valid__.json';
        var url = path.resolve(cwd, 'ts/test/json', nonExistentJsonFile);
        remoteCatalogGetter.getCatalogFromUrl(url, function (err, remoteCatalog) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal("Url: '[object Object]' does not exist");
            chai_1.expect(remoteCatalog).to.not.exist;
            done();
        });
    });
    it('should have callback with error when resource does not contain valid json', function (done) {
        var cwd = process.cwd();
        var nonExistentJsonFile = 'bad-json.json';
        var url = path.resolve(cwd, 'ts/test/json', nonExistentJsonFile);
        remoteCatalogGetter.getCatalogFromUrl(url, function (err, remoteCatalog) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal('Unexpected token : in JSON at position 1');
            chai_1.expect(remoteCatalog).to.not.exist;
            done();
        });
    });
    it('should return error on catalog containing bad url', function (done) {
        var cwd = process.cwd();
        var nonExistentJsonFile = 'badCommandCatalog.json';
        var url = path.resolve(cwd, 'ts/test/json', nonExistentJsonFile);
        remoteCatalogGetter.getCatalogFromUrl(url, function (err, remoteCatalog) {
            chai_1.expect(err).to.exist;
            chai_1.expect(err.message).to.equal("Url: '/home/jreeme/src/firmament-yargs/ts/test/json/prep-ubuntu-16.04-server---.json' does not exist");
            chai_1.expect(remoteCatalog).to.not.exist;
            done();
        });
    });
    it('should return local remoteCatalog', function (done) {
        var cwd = process.cwd();
        var nonExistentJsonFile = 'commandCatalog.json';
        var url = path.resolve(cwd, 'ts/test/json', nonExistentJsonFile);
        remoteCatalogGetter.getCatalogFromUrl(url, function (err, remoteCatalog) {
            chai_1.expect(err).to.not.exist;
            chai_1.expect(remoteCatalog).to.exist;
            done();
        });
    });
    it('should return local remoteCatalog', function (done) {
        var url = 'https://raw.githubusercontent.com/jreeme/firmament-bash/master/command-json/commandCatalog.json';
        remoteCatalogGetter.getCatalogFromUrl(url, function (err, remoteCatalog) {
            chai_1.expect(err).to.not.exist;
            chai_1.expect(remoteCatalog).to.exist;
            done();
        });
    });
});
//# sourceMappingURL=remote-catalog.test.js.map